@@ -1,5 +1,5 @@
 """Module with functions and classes to help with model selection for classification."""
-from biopsykit.classification.model_selection.nested_cv import nested_cv_grid_search
+from biopsykit.classification.model_selection.nested_cv import nested_cv_param_search
 from biopsykit.classification.model_selection.sklearn_pipeline_permuter import SklearnPipelinePermuter
 
-__all__ = ["nested_cv_grid_search", "SklearnPipelinePermuter"]
+__all__ = ["nested_cv_param_search", "SklearnPipelinePermuter"]
