@@ -1,33 +1,30 @@
 """Module containing different I/O functions to load time log data, subject condition lists, questionnaire data, etc."""
 
-from pathlib import Path
-from typing import Optional, Union, Sequence, Dict, Tuple
-
 import datetime
-import pytz
+from pathlib import Path
+from typing import Dict, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import pandas as pd
+import pytz
 from nilspodlib import Dataset
 
-from biopsykit.utils.file_handling import is_excel_file
-from biopsykit.utils.exceptions import ValidationError
-from biopsykit.utils.dataframe_handling import convert_nan
-from biopsykit.utils.time import tz
-
-from biopsykit.utils.datatype_helper import (
-    SubjectConditionDataFrame,
-    SubjectConditionDict,
-    is_subject_condition_dict,
-    is_subject_condition_dataframe,
-    CodebookDataFrame,
-    is_codebook_dataframe,
-    _SubjectConditionDataFrame,
-    _CodebookDataFrame,
-)
-
 from biopsykit.utils._datatype_validation_helper import _assert_file_extension, _assert_has_columns, _assert_is_dtype
 from biopsykit.utils._types import path_t
+from biopsykit.utils.dataframe_handling import convert_nan
+from biopsykit.utils.datatype_helper import (
+    CodebookDataFrame,
+    SubjectConditionDataFrame,
+    SubjectConditionDict,
+    _CodebookDataFrame,
+    _SubjectConditionDataFrame,
+    is_codebook_dataframe,
+    is_subject_condition_dataframe,
+    is_subject_condition_dict,
+)
+from biopsykit.utils.exceptions import ValidationError
+from biopsykit.utils.file_handling import is_excel_file
+from biopsykit.utils.time import tz
 
 __all__ = [
     "load_long_format_csv",
@@ -170,6 +167,7 @@ def load_time_log(
     data, index_cols = _sanitize_index_cols(data, subject_col, condition_col, additional_index_cols)
     data = _apply_index_cols(data, index_cols=index_cols)
     data = _apply_phase_cols(data, phase_cols=phase_cols)
+    data.columns.name = "phase"
 
     if not continuous_time:
         data = _parse_time_log_not_continuous(data, index_cols)
@@ -229,6 +227,9 @@ def _parse_time_log_not_continuous(
     data = data.reindex(["start", "end"], level=-1)
     # unstack start|end level
     data = data.unstack()
+    # set name of outer index level
+    data.columns = data.columns.set_names("phase", level=0)
+
     return data
 
 
