@@ -1,5 +1,5 @@
 """Module for processing saliva data and computing established features (AUC, slope, maximum increase, ...)."""
-from biopsykit.saliva.saliva import auc, slope, initial_value, max_value, max_increase, mean_se, standard_features
 from biopsykit.saliva import utils
+from biopsykit.saliva.saliva import auc, initial_value, max_increase, max_value, mean_se, slope, standard_features
 
 __all__ = ["auc", "initial_value", "max_increase", "max_value", "mean_se", "slope", "standard_features", "utils"]
